'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.read = exports.canConnect = exports.init = exports.account = exports.options = exports.token = exports.contract = exports.reader = exports.writer = exports.tokenCode = exports.ridlCode = undefined;

var _eosjs = require('eosjs');

var _eosjs2 = _interopRequireDefault(_eosjs);

var _Network = require('../models/Network');

var _Network2 = _interopRequireDefault(_Network);

var _bignumber = require('bignumber.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const ridlCode = exports.ridlCode = 'ridlridlridl';
const tokenCode = exports.tokenCode = 'ridlridlcoin';

let writer = exports.writer = null;
let reader = exports.reader = null;
let contract = exports.contract = null;
let token = exports.token = null;
let options = exports.options = {};
let account = exports.account = null;
let net = null;

/***
 * Can be initialized as a reader only.
 * @param network
 * @param _account
 * @param signProvider
 * @returns {Promise.<boolean>}
 */
const init = exports.init = (() => {
    var _ref = _asyncToGenerator(function* (network, _account = null, signProvider = null) {
        network = _Network2.default.fromJson(network);
        net = network;
        if (!!_account) exports.account = account = _account;

        if (!(yield canConnect())) {
            exports.writer = writer = null;
            exports.contract = contract = null;
            exports.token = token = null;
            exports.options = options = {};
            return false;
        }

        exports.reader = reader = (0, _eosjs2.default)({ httpEndpoint: network.fullhost(), chainId: network.chainId });

        if (signProvider) {
            exports.writer = writer = (0, _eosjs2.default)({ httpEndpoint: network.fullhost(), chainId: network.chainId, signProvider });
            exports.contract = contract = yield writer.contract(ridlCode);
            exports.token = token = yield writer.contract(tokenCode);
        } else {
            exports.writer = writer = null;
            exports.contract = contract = null;
            exports.token = token = null;
        }

        const baseOptions = {
            logger: {
                log: null, //console.log,
                error: null //console.error
            }
        };

        exports.options = options = Object.assign(baseOptions, !!_account ? { authorization: [`${_account.name}@${_account.authority}`] } : {});
        return true;
    });

    return function init(_x) {
        return _ref.apply(this, arguments);
    };
})();

/***
 * Checks the connection to the network.
 * @returns {Promise.<*>}
 */
const canConnect = exports.canConnect = (() => {
    var _ref2 = _asyncToGenerator(function* () {
        const timeout = new Promise(function (r) {
            setTimeout(function () {
                return r(false);
            }, 1500);
        });
        const tester = fetch(`${net.fullhost()}/v1/chain/get_info`).then(function () {
            return true;
        }).catch(function () {
            return false;
        });
        return Promise.race([timeout, tester]);
    });

    return function canConnect() {
        return _ref2.apply(this, arguments);
    };
})();

const read = exports.read = (() => {
    var _ref3 = _asyncToGenerator(function* ({ table, index = null, upper_bound = null, nobound = false, limit = 10, model = null, scope = ridlCode, token = false, firstOnly = false, rowsOnly = false, key_type = null, index_position = null, search = null }) {
        let additions = index !== null ? { lower_bound: index, upper_bound: upper_bound ? upper_bound : (0, _bignumber.BigNumber)(index).plus(search !== null ? search : limit).toString() } : {};
        if (nobound) delete additions.upper_bound;
        if (key_type) additions = Object.assign({ key_type }, additions);
        if (index_position) additions = Object.assign({ index_position }, additions);
        return yield reader.getTableRows(Object.assign({ json: true, code: token ? tokenCode : ridlCode, scope, table, limit }, additions)).then(function (result) {
            if (model) result = formatRow(result, model);
            if (firstOnly) return getFirstOnly(result);
            if (rowsOnly) return getRowsOnly(result);
            return result;
        });
    });

    return function read(_x2) {
        return _ref3.apply(this, arguments);
    };
})();

const formatRow = (result, model) => {
    result.rows = result.rows.map(model.fromJson);
    return result;
};

const getRowsOnly = result => result.rows;
const getFirstOnly = result => result.rows.length ? getRowsOnly(result)[0] : null;