"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _eos = require("./eos");

var eos = _interopRequireWildcard(_eos);

var _Reputable = require("../models/Reputable");

var _Reputable2 = _interopRequireDefault(_Reputable);

var _helpers = require("../util/helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

let repTotalCache = {};

const getReputations = (() => {
	var _ref = _asyncToGenerator(function* (reputables) {
		yield Promise.all(reputables.map((() => {
			var _ref2 = _asyncToGenerator(function* (reputable) {
				reputable.reputation = yield getReputation(reputable);
				return true;
			});

			return function (_x2) {
				return _ref2.apply(this, arguments);
			};
		})()));
		repTotalCache = {};
		return true;
	});

	return function getReputations(_x) {
		return _ref.apply(this, arguments);
	};
})();

const getReputation = (() => {
	var _ref3 = _asyncToGenerator(function* (reputable) {
		const reputation = yield eos.read({
			table: 'reputations',
			scope: reputable.id,
			limit: 1,
			firstOnly: true
		}).catch(function () {
			return null;
		});

		const fragTotals = yield Promise.all(reputation.fragments.map((() => {
			var _ref4 = _asyncToGenerator(function* (x) {

				if (repTotalCache.hasOwnProperty(x.fingerprint)) return repTotalCache[x.fingerprint];

				const rep = eos.read({
					table: 'fragtotal',
					scope: x.fingerprint,
					limit: 1,
					firstOnly: true
				}).catch(function () {
					return null;
				});

				if (rep) repTotalCache[x.fingerprint] = rep;
				return rep;
			});

			return function (_x4) {
				return _ref4.apply(this, arguments);
			};
		})()));

		const parseAsset = function (asset) {
			return parseFloat(asset.split(' ')[0]);
		};

		reputation.fragments.map(function (frag) {
			const fragTotal = fragTotals.find(function (x) {
				return x.type === frag.type;
			});
			frag.reputation = 0;

			const up = parseAsset(frag.up);
			const down = parseAsset(frag.down);
			const tup = parseAsset(fragTotal.up);
			const tdown = parseAsset(fragTotal.down);

			// TODO: Might be an issue here.
			frag.reputation = (up > 0 ? up / tup : 0) - (down > 0 ? down / tdown : 0);

			const timeMod = (Math.floor(+new Date() / 1000) - reputable.last_repute_time) / 100000000;
			if (frag.reputation > 0 && frag.reputation - timeMod > frag.reputation / 2) frag.timeScaledReputation = frag.reputation - timeMod;else if (frag.reputation < 0 && frag.reputation + timeMod < frag.reputation / 2) frag.timeScaledReputation = frag.reputation + timeMod;else frag.timeScaledReputation = frag.reputation;

			frag.reputation = parseFloat(frag.reputation).toFixed(4);
			frag.timeScaledReputation = parseFloat(frag.timeScaledReputation).toFixed(4);
			frag.fingerprint = fragTotal.fingerprint;
		});

		delete reputation.id;
		return _Reputable.Reputation.fromJson(reputation);
	});

	return function getReputation(_x3) {
		return _ref3.apply(this, arguments);
	};
})();

const getParents = (() => {
	var _ref5 = _asyncToGenerator(function* (reputable, last = null) {
		if (reputable.base === 0) return true;
		if (last && last.base === 0) return true;

		const parent = yield eos.read({
			table: 'reputables',
			index: last ? last.base : reputable.base,
			limit: 1,
			firstOnly: true,
			model: _Reputable2.default
		}).catch(function () {
			return null;
		});

		if (last) last.parent = parent;else reputable.parent = parent;

		return yield getParents(reputable, parent);
	});

	return function getParents(_x5) {
		return _ref5.apply(this, arguments);
	};
})();

class ReputationService {

	constructor() {}

	repute(username, id = 0, entity, type, fragments, network = "", parent = 0, details = "") {
		return _asyncToGenerator(function* () {
			if (!fragments.every(function (frag) {
				return frag instanceof _Reputable.Fragment && frag.validate();
			})) throw new Error('Invalid fragments');
			return eos.contract.repute(username, id, entity, type, fragments, network, parent, details, eos.options);
		})();
	}

	votetype(username, type) {
		return _asyncToGenerator(function* () {
			return eos.contract.votetype(username, type, eos.options);
		})();
	}

	getEntity(id) {
		return _asyncToGenerator(function* () {

			const reputable = yield eos.read({
				table: 'reputables',
				index: id,
				limit: 1,
				firstOnly: true,
				model: _Reputable2.default
			}).catch(function () {
				return null;
			});

			if (!reputable) return null;

			yield getReputations([reputable]);
			yield getParents(reputable);

			return reputable;
		})();
	}

	searchForEntity(name) {
		return _asyncToGenerator(function* () {

			const reputables = yield eos.read({
				table: 'reputables',
				index: (0, _helpers.fingerprinted)(name),
				key_type: 'i64',
				index_position: 2,
				limit: 500,
				rowsOnly: true,
				model: _Reputable2.default
			}).catch(function () {
				return null;
			});

			if (reputables.length) {
				yield getReputations(reputables);
				yield Promise.all(reputables.map(function (reputable) {
					return getParents(reputable);
				}));
			}

			return reputables;
		})();
	}

	searchByFingerprint(type = '', entity = '', network = '', base = 0) {
		return _asyncToGenerator(function* () {

			const reputable = yield eos.read({
				table: 'reputables',
				index: (0, _helpers.fingerprinted)(type + entity + network + base),
				key_type: 'i64',
				index_position: 3,
				limit: 1,
				firstOnly: true,
				model: _Reputable2.default
			}).catch(function () {
				return null;
			});

			if (!reputable) return null;

			yield getReputations([reputable]);
			yield getParents(reputable);

			return reputable;
		})();
	}

	getFragments(base = 0) {
		return _asyncToGenerator(function* () {
			return eos.read({
				table: 'reptypes',
				key_type: 'i64',
				index_position: 2,
				index: base,
				search: 1,
				limit: 100,
				rowsOnly: true,
				model: _Reputable.RepType
			}).catch(function () {
				return [];
			});
		})();
	}

	getFragmentsFor(reputable = null) {
		var _this = this;

		return _asyncToGenerator(function* () {
			const globalFragments = yield _this.getFragments();
			const basedFragments = reputable ? (yield _this.getFragments(reputable.id)).map(function (x) {
				x.isBased = true;
				return x;
			}) : [];
			return globalFragments.concat(basedFragments);
		})();
	}
}
exports.default = ReputationService;